/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "config.h"
#include "tpchk.h"

#ifndef CORBA_UNIX

#define socket isoc_socket
#define bind isoc_bind
#define sendto isoc_sendto
#define recvfrom isoc_recvfrom
#define close isoc_close

#endif /* CORBA_UNIX */

void
sync_send(char *host, unsigned short port)
{
	int			s;
	int			ret;
	struct sockaddr_in	srv;
	unsigned long		server_addr;
	char			cmd;

	s = socket(PF_INET, SOCK_DGRAM, 0);
	TP_CHECK((s >= 0), (" "));

	server_addr = TP_get_ipaddr(host);

	srv.sin_family = PF_INET;
	srv.sin_port = htons(port);
	srv.sin_addr.s_addr = htonl((unsigned long)server_addr);
	bzero(srv.sin_zero, 8);

	TP_Msg("- sync_send -\n");
  	ret = sendto(s, (char *)&cmd, sizeof(CORBA_int), 0,
			(struct sockaddr *)&srv, sizeof(srv));
	TP_CHECK((ret >= 0), ("%d", ret));

	close(s);
}

void
sync_recv(unsigned short port)
{
	CORBA_int		cmd;
	int			s;
	int			namelen;
	int			ret;
	struct sockaddr_in	srv;
	struct sockaddr_in	cli;

	s = socket(PF_INET, SOCK_DGRAM, 0);

	srv.sin_family = PF_INET;
	srv.sin_port = htons(port);
	srv.sin_addr.s_addr = htonl(0);
	bzero(srv.sin_zero, 8);
	ret = bind(s, (struct sockaddr *)&srv, sizeof(srv));
	TP_CHECK((ret >= 0), (" "));

	namelen = sizeof(cli);
	TP_Msg("- sync_recv wait -\n");
	ret = recvfrom(s, (char *)&cmd, sizeof(CORBA_int), 0,
			(struct sockaddr *)&cli, &namelen);
	TP_Msg("- sync_recv end -\n");
	TP_CHECK((ret >= 0), (" "));

	close(s);
}

CORBA_int
tsk_stop_check(ID id, char *host, unsigned short port)
{
	int			s;
	int			ret;
	int			namelen;
	struct sockaddr_in	srv;
	struct sockaddr_in	from;
	unsigned long		server_addr;
	CORBA_int		retval;

	TP_Msg("Server Task Stop Check (id: %d)\n", id);

	s = socket(PF_INET, SOCK_DGRAM, 0);
	TP_CHECK((s >= 0), (" "));

	server_addr = TP_get_ipaddr(host);

	srv.sin_family = PF_INET;
	srv.sin_port = htons(port);
	srv.sin_addr.s_addr = htonl((unsigned long)server_addr);
	bzero(srv.sin_zero, 8);

  	ret = sendto(s, (char *)&id, sizeof(ID), 0,
			(struct sockaddr *)&srv, sizeof(srv));
	TP_CHECK((ret >= 0), ("%d", ret));

	namelen = sizeof(from);
	ret = recvfrom(s, (char *)&retval, sizeof(CORBA_int), 0,
			(struct sockaddr *)&from, &namelen);
	TP_CHECK((ret >= 0), ("%d", ret));

	close(s);
	TP_Msg("Server Task Stop OK (id: %d)\n", id);

	return retval;
}

void
tsk_stop_wait(unsigned short port)
{
	CORBA_int		retval;
	ID			id;
	int			s;
	int			namelen;
	int			ret;
	struct sockaddr_in	srv;
	struct sockaddr_in	cli;
	ER			ercd;
	T_RTSK			pk_rtsk;

	s = socket(PF_INET, SOCK_DGRAM, 0);
	TP_CHECK((s >= 0), (" "));

	srv.sin_family = PF_INET;
	srv.sin_port = htons(port);
	srv.sin_addr.s_addr = htonl(0);
	bzero(srv.sin_zero, 8);
	ret = bind(s, (struct sockaddr *)&srv, sizeof(srv));
	TP_CHECK((ret >= 0), (" "));

	namelen = sizeof(cli);
	ret = recvfrom(s, (char *)&id, sizeof(ID), 0,
			(struct sockaddr *)&cli, &namelen);
	TP_CHECK((ret >= 0), (" "));
	TP_Msg("Server Task Stop Wait id: %d\n", id);

	while (1) {
		ercd = ref_tsk(&pk_rtsk, id);
		if (ercd == E_OK) {
			tslp_tsk(10);
		} else {
			tslp_tsk(1);
			break;
		}
	};

  	ret = sendto(s, (char *)&retval, sizeof(CORBA_int), 0,
			(struct sockaddr *)&cli, sizeof(srv));
	TP_CHECK((ret >= 0), ("%d", ret));

	close(s);
	TP_Msg("Server Task Stop id: %d\n", id);
}
