/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "config.h"
#include "tpchk.h"

CORBA_int	TP_ExcFlg;

static CORBA_UsrDefine corba_def = {
	CORBA_TARGET_NAME,
	CORBA_SERVER_NUMBER,			
	CORBA_MAX_MEMORY,
	CORBA_NULL,
	0,
	0,
	0,
	0
};

static ASRV_UsrDefine asrv_def = {
	ASRV_SERVER_ID,		
	ASRV_SERVER_PRI,		
	ASRV_OBJECT_PRI,		
	ASRV_SERVER_PORT,
	ASRV_OBJECT_NUMBER,		
	ASRV_DYNAMIC_NUMBER,		
	0,
	0,
	0,
	0
};

static ORB_UsrDefine orb_def;

static POA_UsrDefine poa_def = {
	POA_MAX_POA_NUMBER,		
	POA_MAX_SERVANT_NUMBER,		
	0,
	0,
	0,
	0
};

static MRSHL_UsrDefine mrshl_def;
static GIOP_UsrDefine giop_def;

static CON_UsrDefine con_def = {
	CON_SERVER_NUMBER,		
	CON_CLIENT_NUMBER,		
	CON_SERVER_TIMEOUT,
	0,
	0,
	0,
	0
};

static SYS_UsrDefine sys_def;

#define WAITCNT	0x1000
static CORBA_int WaitCnt;

CORBA_int
TP_wait(CORBA_int init)
{
	CORBA_int	i;

	for (i = 1, WaitCnt = init; i < WAITCNT; i++) {
		WaitCnt *= i;
	}
	return WaitCnt;
}

void
TP_corba_init(void)
{
	SG_CORBA	sg;
	CORBA_int	ret;

	sg.sg_corba = &corba_def;
	sg.sg_asrv = &asrv_def;
	sg.sg_orb = &orb_def;
	sg.sg_poa = &poa_def;
	sg.sg_mrshl = &mrshl_def;
	sg.sg_giop = &giop_def;
	sg.sg_con = &con_def;
	sg.sg_sys = &sys_def;

	ret = ECORBA_init(&sg);
	TP_CHECK((ret == CORBA_SGOK), ("ret:%d ", ret));
}

void
TP_corba_init2(CORBA_unsigned_short port)
{
	SG_CORBA	sg;
	CORBA_int	ret;
	ASRV_UsrDefine	asrv_def2;

	TP_Msg("asrv port number: %d\n", port);

	asrv_def2.asrv_aservid = ASRV_SERVER_ID;
	asrv_def2.asrv_asrvpri = ASRV_SERVER_PRI;
	asrv_def2.asrv_dsrvpri = ASRV_OBJECT_PRI;
	asrv_def2.asrv_portnum = port;
	asrv_def2.asrv_dservnm = ASRV_OBJECT_NUMBER;
	asrv_def2.asrv_ddobjnm = ASRV_DYNAMIC_NUMBER;

	sg.sg_corba = &corba_def;
	sg.sg_asrv = &asrv_def2;
	sg.sg_orb = &orb_def;
	sg.sg_poa = &poa_def;
	sg.sg_mrshl = &mrshl_def;
	sg.sg_giop = &giop_def;
	sg.sg_con = &con_def;
	sg.sg_sys = &sys_def;

	ret = ECORBA_init(&sg);
	TP_CHECK((ret == CORBA_SGOK), ("ret:%d ", ret));
}

void
TP_corba_fini(void)
{
	ASRV_fini();
	MRSHL_fini();
	CON_fini();
	GIOP_fini();
	ORB_fini();
	POA_fini();
}

#ifndef	CORBA_UNIX

void
TP_Msg(CORBA_char *fmt, ...)
{
	CORBA_char	buf[100];

	va_list	ap;
	va_start(ap, fmt);

	vsprintf(buf, fmt, ap);
	_PutString(buf);
	va_end(ap);
	TP_wait(1);
}

WERR
TP_CreateTask(FP entry, PRI pri)
{
        ERR     er;
        ID      tskid;
        T_CTSK  ctsk;

        ctsk.exinf   = (VP)CH4toW('-','t','p','-');
        ctsk.task    = entry;
        ctsk.itskpri = pri;
        ctsk.stksz   = TASK_STKSZ;
        ctsk.tskatr  = TA_HLNG|TA_RNG0;
        er = vcre_tsk(&ctsk);
	TP_CHECK((er >= E_OK), ("er: %d", er));

	tskid = (ID)er;
        er = sta_tsk(tskid, 0);
	TP_CHECK((er >= E_OK), ("er: %d", er));

        return tskid;
}

void
TP_delete_ASRVtsk(ID id)
{
	T_RTSK	pk_rtsk;
	ER	ercd;

	ercd = ref_tsk(&pk_rtsk, id);
	if (ercd != E_OK) {
		return;
	}
	ter_tsk(id);
	del_tsk(id);
}

CORBA_unsigned_short
TPswaps(CORBA_unsigned_short port)
{
	CORBA_boolean	flg;

	flg = MRSHL_get_native_order();
	if (flg == CORBA_FALSE) {
		return port;
	} else {
		return (((port >> 8) & 0xff) | ((port & 0xff) << 8));
	}
}

CORBA_unsigned_long
TPswapl(CORBA_unsigned_long ipaddr)
{
	CORBA_boolean	flg;

	flg = MRSHL_get_native_order();
	if (flg == CORBA_FALSE) {
		return ipaddr;
	} else {
		return (((ipaddr & 0xff000000) >> 24)
			| ((ipaddr & 0x00ff0000) >> 8)
			| ((ipaddr & 0x0000ff00) << 8) 
			| ((ipaddr & 0x000000ff)<< 24));
	}
}

#endif	/* CORBA_UNIX */

#define SYS_MEMHDR	(8)

CORBA_boolean
tp_check_sysmem(CORBA_int presz, CORBA_int cursz, CORBA_int getsz,
		CORBA_int count)
{
	getsz += ((SYS_MEMHDR + sizeof(CORBA_long_long)) * count);
	if ((presz - cursz) == getsz) {
		return CORBA_TRUE;
	} else {
		return CORBA_FALSE;
	}
}

#define CORBA_MEMINF_SZ	(16)

CORBA_boolean
tp_check_corbamem(CORBA_int presz, CORBA_int cursz, CORBA_int getsz,
		  CORBA_int count)
{
	getsz += ((CORBA_MEMINF_SZ + SYS_MEMHDR + sizeof(CORBA_long_long)) * count);
	if ((presz - cursz) == getsz) {
		return CORBA_TRUE;
	} else {
		return CORBA_FALSE;
	}
}

