/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "config.h"
#include "tpchk.h"
#include "s2105.h"
#include "_sysdbg.h"
#include "_loging.h"

static void func(CORBA_Object, CORBA_long, CORBA_Environment *);

#define IOR_BUFSZ	(1000)
CORBA_char	IOR_str[IOR_BUFSZ];

int		argc;
char		*argv = "";

void
task(void)
{
	CORBA_Environment	ev;
	CORBA_ORB		orb;
	CORBA_Object		obj;

	TP_START();
	IPA_LOG_init(0);

	TP_corba_init();
	TP_IOR_recv(IOR_RECV_PORT, IOR_str);

	CORBA_exception_init(&ev);

	orb = CORBA_ORB_init(&argc, &argv, "e-corba", &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	obj = CORBA_ORB_string_to_object(orb, IOR_str, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((obj != CORBA_NULL), (" "));

	TP_LINE();
	TP_INFO(("s210701\n"));
	{
		CORBA_exception_init(&ev);

		IPA_LOG_init(0);

		func(obj, EXCEPTION_NO, &ev);
		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
		TP_Msg("_major : %d\n", ev._major);

		IPA_LOG_init(0);

		CORBA_exception_free(&ev);
	}
	TP_INFO(("s210701 OK\n"));
	TP_LINE();

	TP_INFO(("s210702\n"));
	TP_INFO(("s210703\n"));
	{
		CORBA_char		*id;
		CORBA_SystemException	*value;
		CORBA_int		i;

		for (i = 1; i < 29; i++) {
			CORBA_exception_init(&ev);
			func(obj, EXCEPTION_SYSTEM, &ev);
			TP_CHECK((ev._major == CORBA_SYSTEM_EXCEPTION), (" "));
			TP_Msg("_major : %d\n", ev._major);

			id = CORBA_exception_id(&ev);
			TP_CHECK((strcmp(id, Exc[i]) == 0), (" "));
			TP_Msg("_id : %s\n", id);

			value = CORBA_exception_value(&ev);
			TP_CHECK((value != CORBA_NULL), (" "));
			TP_Msg("_completed : %d\n", value->_completed);

			CORBA_exception_free(&ev);
		}
	}
	TP_INFO(("s210702 OK\n"));
	TP_INFO(("s210703 OK\n"));
	TP_LINE();

	TP_INFO(("s210704\n"));
	TP_INFO(("s210705\n"));
	{
		CORBA_char	*id;
		errexc		*err;

		CORBA_exception_init(&ev);
		func(obj, EXCEPTION_USER, &ev);
		TP_CHECK((ev._major == CORBA_USER_EXCEPTION), (" "));
		TP_Msg("_major : %d\n", ev._major);

		id = CORBA_exception_id(&ev);
		TP_CHECK((id != CORBA_NULL), (" "));
		TP_Msg("_id : %s\n", id);

		err = CORBA_exception_value(&ev);
		TP_CHECK((err != CORBA_NULL), (" "));
		TP_CHECK((err->code == ERREXC_CODE), (" "));
		TP_Msg("err->code : %d\n", err->code);

		CORBA_exception_free(&ev);
	}
	TP_INFO(("s210704 OK\n"));
	TP_INFO(("s210705 OK\n"));
	TP_LINE();

	/* server shutdown */
	{
		CORBA_exception_init(&ev);
		func(obj, SHUTDOWN_CODE, &ev);
	}

	CORBA_Object_release(obj, &ev);
	CORBA_Object_release((CORBA_Object)orb, &ev);

	TP_corba_fini();
	TP_END();
}

static void
func(CORBA_Object obj, CORBA_long dat, CORBA_Environment *ev)
{
	CORBA_boolean		order;
	CORBA_char		*topp;
	CORBA_char		*bufp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	GIOP_init_cli(&cliinf);

	do {
		size = 0;
		size = MRSHL_index_long(size);
		topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
					"func", ev);
		TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

		bufp = topp;
		MRSHL_put_long(&bufp, dat);
		size = bufp - topp;
		topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	} while (ev->_flag == ENVFLG_LOCATION_FORWARD);

	bufp = topp;
	order = GIOP_cli_bordr(&cliinf);
	switch (ev->_major) {
		errexc	*err;

	case CORBA_NO_EXCEPTION:
		break;
	case CORBA_SYSTEM_EXCEPTION:
		break;
	case CORBA_USER_EXCEPTION:
		err = errexc__alloc();
		if (!err) {
			CORBA_exception_set(ev, CORBA_SYSTEM_EXCEPTION,
					    CORBA_Ex_NO_MEMORY, 0);
		} else {
			err->code = MRSHL_get_long(&bufp, order);
			CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
					    ex_errexc, err);
		}
	}
	GIOP_end_cli(&cliinf);
}

#ifdef	CORBA_UNIX

int
main(void)
{
	MAIN_START();	

	SYS_init_debug(0);
	task();

	MAIN_END();	
	return 0;

}

#else	/* CORBA_UNIX */

ERR
main(Bool Startup, TC *arg)
{
	MAIN_START();	

	SYS_init_debug(0);
	IPA_LOG_init(0);

	TP_CreateTask(task, DEFAULT_PRI);

	MAIN_END();
	return 0;
}

#endif	/* CORBA_UNIX */

