/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "config.h"
#include "tpchk.h"
#include "s2101.h"
#include "_loging.h"
#include "_sysdbg.h"

static void null_func(CORBA_Object, CORBA_Environment *);
static CORBA_char one_func(CORBA_Object, CORBA_char, CORBA_Environment *);
static test_K_slice *K_func(CORBA_Object, test_K, CORBA_Environment *);
static void buff_mem_error_func(CORBA_Object, CORBA_Environment *);
static void mget_func(CORBA_Object, CORBA_Environment *);
static void mfree_func(CORBA_Object, CORBA_Environment *);
static void memory_error_happen_func(CORBA_Object, CORBA_Environment *);
static void shutdown_func(CORBA_Object, CORBA_Environment *);

CORBA_char IOR_str[1000];
static test_K	Kdata;

void
task(void)
{
	int			argc = 0;
	char			*argv = "";
	CORBA_Environment	ev;
	CORBA_ORB		orb;
	CORBA_Object		obj;

	TP_START();

	TP_corba_init();
	TP_IOR_recv(IOR_RECV_PORT, IOR_str);

	TP_Msg("env init\n");
	CORBA_exception_init(&ev);

	TP_Msg("CORBA_ORB_init\n");
	orb = CORBA_ORB_init(&argc, &argv, "e-corba", &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("CORBA_ORB_string_to_object\n");
	obj = CORBA_ORB_string_to_object(orb, IOR_str, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((obj != CORBA_NULL), (" "));

	TP_LINE();
	TP_INFO(("s210101\n"));
	TP_INFO(("s210102\n"));
	{
		CORBA_exception_init(&ev);
		null_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
		TP_Msg("null_func OK\n");
	}
	TP_INFO(("s210101 OK\n"));
	TP_INFO(("s210102 OK\n"));

	TP_LINE();
	TP_INFO(("s210103\n"));
	TP_INFO(("s210104\n"));
	{
		CORBA_char	ret;

		CORBA_exception_init(&ev);

		IPA_LOG_init(0);

		ret = one_func(obj, ONE_BYTE_DATA, &ev);

		IPA_LOG_init(0);

		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

		TP_CHECK((ret == RET_ONE_BYTE_DATA), (" "));
		TP_Msg("retval dat 0x%x\n", ret);
		TP_Msg("one_byte_data_func OK\n");
	}
	TP_INFO(("s210103 OK\n"));
	TP_INFO(("s210104 OK\n"));

	TP_LINE();
	TP_INFO(("s210105\n"));
	TP_INFO(("s210106\n"));
	{
		CORBA_long	i;
		test_K_slice	*ret;

		for (i = 0; i < K_DATA_SIZE; i++) {
			Kdata[i] = i;
		}

		CORBA_exception_init(&ev);
		ret = K_func(obj, Kdata, &ev);

		for (i = 0; i < K_DATA_SIZE; i++) {
			TP_CHECK((ret[i] == K_DATA_SIZE - i), (" "));
		}
		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
		CORBA_free(ret);
		TP_Msg("K_func OK\n");
	}
	TP_INFO(("s210105 OK\n"));
	TP_INFO(("s210106 OK\n"));

	TP_LINE();
	TP_INFO(("s210107\n"));
	{
		CORBA_char		*id;
		CORBA_SystemException	*value;

		CORBA_exception_init(&ev);
		buff_mem_error_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_SYSTEM_EXCEPTION), (" "));
		TP_Msg("_major : %d\n", ev._major);
		
		id = CORBA_exception_id(&ev);
		TP_CHECK((id != CORBA_NULL), (" "));
		TP_CHECK((strcmp(id, CORBA_Ex_NO_MEMORY) == 0), (" "));
		TP_Msg("_id : %s\n", id);

		value = CORBA_exception_value(&ev);
		TP_CHECK((value != CORBA_NULL), (" "));
		TP_CHECK((value->_completed == CORBA_COMPLETED_NO), (" "));
		TP_Msg("_completed %d\n", value->_completed);

		CORBA_exception_free(&ev);
		TP_Msg("buff_mem_error_func OK\n");
	}
	TP_INFO(("s210107 OK\n"));

	TP_LINE();
	TP_INFO(("s210108\n"));
	{
		CORBA_char		*id;
		CORBA_SystemException	*value;

		CORBA_exception_init(&ev);
		mget_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

		memory_error_happen_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_SYSTEM_EXCEPTION), (" "));
		TP_Msg("_major : %d\n", ev._major);
		
		id = CORBA_exception_id(&ev);
		TP_CHECK((id != CORBA_NULL), (" "));
		TP_CHECK((strcmp(id, CORBA_Ex_NO_MEMORY) == 0), (" "));
		TP_Msg("_id : %s\n", id);

		value = CORBA_exception_value(&ev);
		TP_CHECK((value != CORBA_NULL), (" "));
		TP_CHECK((value->_completed == CORBA_COMPLETED_NO), (" "));
		TP_Msg("_completed %d\n",	value->_completed);

		CORBA_exception_free(&ev);

		mfree_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

		TP_Msg("memory_error_happen_func OK\n");
	}
	TP_INFO(("s210108 OK\n"));

	CORBA_exception_init(&ev);
	shutdown_func(obj, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	CORBA_Object_release(obj, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	obj = CORBA_ORB_string_to_object(orb, IOR_str, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((obj != CORBA_NULL), (" "));

	TP_LINE();
	TP_INFO(("s210109\n"));
	{
		CORBA_char		*id;

		null_func(obj, &ev);
		TP_CHECK((ev._major == CORBA_SYSTEM_EXCEPTION), (" "));
		TP_Msg("_major : %d\n", ev._major);

		id = CORBA_exception_id(&ev);
		TP_CHECK((id != CORBA_NULL), (" "));
		TP_CHECK((strcmp(id, CORBA_Ex_NO_RESOURCES) == 0), (" "));
		TP_Msg("_id : %s\n", id);

		CORBA_exception_free(&ev);
		TP_Msg("connection error OK\n");
	}
	TP_INFO(("s210109 OK\n"));

	CORBA_Object_release(obj, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	CORBA_Object_release((CORBA_Object)orb, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	TP_corba_fini();
	TP_delete_ASRVtsk(ASRV_SERVER_ID);

	TP_END();
}

static void
null_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"null_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("null_func invoke\n");
	size = 0;
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	TP_Msg("null_func invoke end\n");

	GIOP_end_cli(&cliinf);
}

static CORBA_char
one_func(CORBA_Object obj, CORBA_char dat, CORBA_Environment *ev)
{
	CORBA_boolean		order;
	CORBA_char		*topp;
	CORBA_char		*bufp;
	CORBA_char		ret;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	size = MRSHL_index_char(size);
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"one_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	bufp = topp;
	MRSHL_put_char(&bufp, dat);
	size = bufp - topp;
	TP_Msg("one_func invoke\n");
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	bufp = topp;
	order = GIOP_cli_bordr(&cliinf);
	ret = MRSHL_get_char(&bufp, order);
	TP_Msg("one_func invoke end\n");

	GIOP_end_cli(&cliinf);

	return ret;
}

test_K_slice *
test_K__alloc()
{
	return (test_K_slice *)CORBA_malloc(sizeof(test_K), 1, 0);
}

static test_K_slice *
K_func(CORBA_Object obj, test_K dat, CORBA_Environment *ev)
{
	CORBA_boolean		order;
	CORBA_char		*topp;
	CORBA_char		*bufp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;
	CORBA_int		i;
	test_K_slice		*ret;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	size = (CORBA_unsigned_long)sizeof(test_K);
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"K_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	bufp = topp;
	MRSHL_bcopy(&bufp, (CORBA_char *)dat, sizeof(CORBA_long),
		    (CORBA_unsigned_long)sizeof(test_K));
	size = bufp - topp;
	TP_Msg("K_func invoke\n");
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	bufp = topp;
	order = GIOP_cli_bordr(&cliinf);
	ret = test_K__alloc();
	TP_CHECK((ret != CORBA_NULL), (" "));
	for (i = 0; i < K_DATA_SIZE; i++) {
		ret[i] = MRSHL_get_long(&bufp, order);
	}

	TP_Msg("K_func invoke end\n");
	GIOP_end_cli(&cliinf);

	return ret;
}

static void
buff_mem_error_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0x7fffffff;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"buff_mem_error_func", ev);
	TP_CHECK((ev->_major != CORBA_NO_EXCEPTION), (" "));
	GIOP_end_cli(&cliinf);
}

static void
mget_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"mget_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("mget_func invoke\n");
	size = 0;
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	TP_Msg("mget_func invoke end\n");

	GIOP_end_cli(&cliinf);
}

static void
mfree_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"mfree_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("mfree_func invoke\n");
	size = 0;
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	TP_Msg("mfree_func invoke end\n");

	GIOP_end_cli(&cliinf);
}

static void
memory_error_happen_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 1000;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"memory_error_happen_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("memory_error_happen_func invoke\n");
	size = 1000;
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	TP_Msg("memory_error_happen_func invoke end\n");

	GIOP_end_cli(&cliinf);
}

static void
shutdown_func(CORBA_Object obj, CORBA_Environment *ev)
{
	CORBA_char		*topp;
	CORBA_unsigned_long	size;
	GIOP_CliInf		cliinf;

	CORBA_exception_init(ev);
	GIOP_init_cli(&cliinf);

	size = 0;
	topp = GIOP_create_reqbuf(obj, &cliinf, size, CORBA_TRUE,
				"shutdown_func", ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_Msg("shutdown_func invoke\n");
	size = 0;
	topp = GIOP_invoke_req(obj, &cliinf, size, ev);
	TP_Msg("shutdown_func invoke end\n");

	GIOP_end_cli(&cliinf);
}

#ifdef	CORBA_UNIX

int
main(void)
{
	MAIN_START();	

	SYS_init_debug(0);
	task();

	MAIN_END();	
	return 0;
}

#else	/* CORBA_UNIX */

ERR
main(Bool Startup, TC *arg)
{
	MAIN_START();	

	SYS_init_debug(0);
	IPA_LOG_init(0);

	TP_CreateTask(task, DEFAULT_PRI);

	MAIN_END();

	return 0;
}

#endif	/* CORBA_UNIX */

