/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "config.h"
#include "tpchk.h"
#include "s2101.h"
#include "_loging.h"
#include "_sysdbg.h"

static void _skl_dispatch_test(PortableServer_Servant, GIOP_SrvInf *);

CORBA_ORB	Orb;
CORBA_char	*Memp;

static void
null_func_impl(PortableServer_Servant s, CORBA_Environment *e)
{
	TP_LINE();

	TP_Msg("s210101\n");
	TP_Msg("s210102\n");
	TP_Msg("null_func_impl OK\n");

	TP_LINE();
}

static CORBA_char
one_func_impl(PortableServer_Servant s, CORBA_char dat, CORBA_Environment *e)
{
	TP_LINE();

	TP_Msg("s210103\n");
	TP_Msg("s210104\n");
	TP_CHECK((dat == ONE_BYTE_DATA), (" "));
	TP_Msg("one_func_impl recv one byte data OK: 0x%x\n", dat & 0xff);

	TP_LINE();
	return RET_ONE_BYTE_DATA;
}

test_K_slice *
test_K__alloc()
{
	return (test_K_slice *)CORBA_malloc(sizeof(test_K), 1, 0);
}

static test_K_slice *
K_func_impl(PortableServer_Servant s, test_K dat, CORBA_Environment *e)
{
	CORBA_long	i;
	test_K_slice	*ret;

	TP_LINE();
	TP_Msg("s210105\n");
	TP_Msg("s210106\n");

	for (i = 0; i < K_DATA_SIZE; i++) {
		TP_CHECK((dat[i] == i), (" "));
	}

	ret = test_K__alloc();
	TP_CHECK((ret != CORBA_NULL), (" "));

	for (i = 0; i < K_DATA_SIZE; i++) {
		ret[i] = K_DATA_SIZE - i;
	}

	TP_Msg("K_func_impl recv OK\n");

	TP_LINE();
	return ret;
}

static void
mget_func_impl(PortableServer_Servant s, CORBA_Environment *e)
{
	CORBA_unsigned_long	size;

	for (size = CORBA_MAX_MEMORY, Memp = CORBA_NULL;
	     Memp == CORBA_NULL;
	     size -= sizeof(CORBA_long)) {
	  	Memp = (CORBA_char *)CORBA_malloc(size, 1, 0);
	}
	TP_Msg("get memory size : 0x%x\n", size);
}

static void
mfree_func_impl(PortableServer_Servant s, CORBA_Environment *e)
{
	CORBA_free(Memp);
}


static void
shutdown_func_impl(PortableServer_Servant s, CORBA_Environment *e)
{
	CORBA_ORB_shutdown(Orb, e);
}

static PortableServer_ServantBase__epv base_epv = {
	NULL,
	NULL,
	NULL
};

typedef struct tagPOA_Test__epv {
	void	*_private;
	void (*func1)(PortableServer_Servant, CORBA_Environment *);
	CORBA_char (*func2)(PortableServer_Servant, CORBA_char, CORBA_Environment *);
	test_K_slice *(*func3)(PortableServer_Servant, test_K, CORBA_Environment *);
	void (*func4)(PortableServer_Servant, CORBA_Environment *);
	void (*func5)(PortableServer_Servant, CORBA_Environment *);
	void (*func6)(PortableServer_Servant, CORBA_Environment *);
} POA_Test__epv;

typedef struct POA_Test__vepv {
	void	*_private;
	PortableServer_ServantBase__epv	*_base_epv;
	POA_Test__epv	*Test_epv;
} POA_Test__vepv;

typedef struct tagPOA_Test {
	void		*_private;
	POA_Test__vepv	*vepv;
} POA_Test;

static PortableServer_ServantPrivateInfo test_class = {
	"test",
	(void (*)())_skl_dispatch_test
};

static void
POA_Test__init(POA_Test *servant, CORBA_Environment *env)
{
	POA_Test	*test;
	
	test = (POA_Test *)servant;
	((PortableServer_ServantBase *)test)->vepv->_private 
					= (void *)&test_class;
	/*
	PortableServer_ServantBase__init(
		(PortableServer_ServantBase*)servant,
		env
	);
	*/
}


void
task(void)
{
	int		argc = 0;
	char		*argv = "";
	CORBA_Environment ev;
	CORBA_Object obj;

	PortableServer_ObjectId	*oid;
	PortableServer_POA root_poa;
	PortableServer_POAManager poa_mgr = NULL;
	POA_Test__epv test_epv = {
	  	NULL,
		null_func_impl,
		one_func_impl,
		K_func_impl,
		mget_func_impl,
		mfree_func_impl,
		shutdown_func_impl
	};
	POA_Test__vepv test_vepv = {NULL, &base_epv, &test_epv};
	POA_Test test = {NULL, &test_vepv };
	CORBA_char *str;

	TP_START();

	CORBA_exception_init(&ev);

	Orb = CORBA_ORB_init(&argc, &argv, "e-corba", &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
         * RootPOA롣
         */
	obj = CORBA_ORB_resolve_initial_references(Orb, "RootPOA", &ev);
	root_poa = (PortableServer_POA)obj;
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
         * RootPOAȴϢŤ줿POAManager롣
         */
	poa_mgr = PortableServer_POA__get_the_POAManager(root_poa, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	POA_Test__init(&test, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	
	oid = PortableServer_string_to_ObjectId("test", &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	
	/*
         * POA˥Х(bank_account)Ͽ롣
         */
	PortableServer_POA_activate_object_with_id(root_poa, oid, &test, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
         * POAManager˳Υå롣
         * POAManagerϿƤ뤹٤ƤPOA롣
         */
	PortableServer_POAManager_activate(poa_mgr, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
	 * ХȤ饪֥ȥե󥹤κ
         */
        obj = PortableServer_POA_servant_to_reference(root_poa, &test, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));
	
	/*
	 * ֥ȥե󥹤ʸѴ
         */
	str = CORBA_ORB_object_to_string(Orb, obj, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
	 * ֥ȥե󥹤
         * 饤ȤذϤ
         */
	TP_IOR_send(IOR_RECV_PORT, str);

	IPA_LOG_Srv_init(0);
	/*
	 * ORBå롼פ롣
         */
	CORBA_ORB_run(Orb, &ev);

	PortableServer_POA_deactivate_object(root_poa, oid, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	/*
	 * POA
         */
	PortableServer_POA_destroy(root_poa, CORBA_TRUE, CORBA_TRUE, &ev);
	TP_CHECK((ev._major == CORBA_NO_EXCEPTION), (" "));

	TP_END();
}

static test_K Kdat;

static void
_skl_dispatch_test(PortableServer_Servant servant, GIOP_SrvInf *msg)
{
	CORBA_boolean		order;
	CORBA_char		*bufp;
	CORBA_Environment	ev;

	bufp = GIOP_srv_bcurp(msg);
	order = GIOP_srv_bordr(msg);

	TP_Msg("\n_skl_dispatch_test\n");
	TP_Msg("opname %s\n", GIOP_srv_opnam(msg));
	CORBA_exception_init(&ev);

	if (!strcmp(GIOP_srv_opnam(msg), "null_func")) {
		CORBA_int	size;

		((POA_Test *)servant)->vepv->Test_epv->func1(servant, &ev);
		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);

	} else if (!strcmp(GIOP_srv_opnam(msg), "one_func")) {
		CORBA_int		size;
		CORBA_char		dat;
		CORBA_char		ret;

		IPA_LOG_Srv_init(0);	

		dat = MRSHL_get_char(&bufp, order);
		ret = ((POA_Test *)servant)->vepv->Test_epv->func2(servant, dat, &ev);
		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		MRSHL_put_char(&bufp, ret);
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);

	} else if (!strcmp(GIOP_srv_opnam(msg), "K_func")) {
		CORBA_int	size;
		CORBA_long	i;	
		test_K_slice	*ret;

		for (i = 0; i < K_DATA_SIZE; i++) {
			Kdat[i] = MRSHL_get_long(&bufp, order);
		}
		ret = ((POA_Test *)servant)->vepv->Test_epv->func3(servant, Kdat, &ev);
		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		MRSHL_bcopy(&bufp, (CORBA_char *)ret, sizeof(CORBA_long),
			    (CORBA_unsigned_long)sizeof(test_K));
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);

	} else if (!strcmp(GIOP_srv_opnam(msg), "mget_func")) {
		CORBA_int	size;

		((POA_Test *)servant)->vepv->Test_epv->func4(servant, &ev);
		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);
	} else if (!strcmp(GIOP_srv_opnam(msg), "memory_error_happen_func")) {
		TP_Msg("memory_error_happen_func\n");
		TP_CHECK(0, (""));

	} else if (!strcmp(GIOP_srv_opnam(msg), "mfree_func")) {
		CORBA_int	size;

		((POA_Test *)servant)->vepv->Test_epv->func5(servant, &ev);

		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);

	} else if (!strcmp(GIOP_srv_opnam(msg), "shutdown_func")) {
		CORBA_int	size;

		((POA_Test *)servant)->vepv->Test_epv->func6(servant, &ev);

		size = 0;
		bufp = GIOP_create_repbuf(msg, size);
		TP_CHECK((bufp != CORBA_NULL), (" "));

		MRSHL_put_long(&bufp, CORBA_NO_EXCEPTION);
		size = (CORBA_unsigned_long)(bufp - GIOP_srv_btopp(msg));
		GIOP_send_rep(msg, size);

	} else {
		TP_CHECK((0), (" "));
	}

}

#ifdef	CORBA_UNIX

int
main(void)
{
	MAIN_START();	

	TP_corba_init();

	SYS_init_debug(0);

	task();

	MAIN_END();	
	TP_corba_fini();

	return 0;
}

#else	/* CORBA_UNIX */

ERR
main(Bool Startup, TC *arg)
{
	MAIN_START();	

	TP_corba_init();

	SYS_init_debug(0);
	IPA_LOG_init(0);	

	TP_CreateTask(task, DEFAULT_PRI);

	MAIN_END();
	TP_corba_fini();

	return 0;
}

#endif	/* CORBA_UNIX */
