/*--------------------------------------------------------------*/
/*      Copyright (c) 2000 NEC Corporation                      */
/*      NEC CONFIDENTIAL AND PROPRIETARY                        */
/*      All rights reserved by NEC Corporation.                 */
/*      This program must be used solely for the purpose for    */
/*      which it was furnished by NEC Corporation.   No part    */
/*      of this program may be reproduced  or  disclosed  to    */
/*      others,  in  any form,  without  the  prior  written    */
/*      permission of NEC Corporation.    Use  of  copyright    */
/*      notice does not evidence publication of the program.    */
/*--------------------------------------------------------------*/
#include "s2104.h"
#include "s2104dat.h"
#include "config.h"
#include "corba.h"
#include "_sysdbg.h"	/* ### for logging message */
#include "_loging.h"

CORBA_char	IORstr[1024];

static void
char_invoke(Test test, CORBA_char dat, CORBA_Environment *ev)
{
	CORBA_char		ret;

	printf("request data = %d\n", (CORBA_int)dat);
	ret = test_func_char(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %d\n", (CORBA_int)ret);
}

static void
short_invoke(Test test, CORBA_short dat, CORBA_Environment *ev)
{
	CORBA_short		ret;

	printf("request data = %d\n", (CORBA_int)dat);
	ret = test_func_short(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %d\n", (CORBA_int)ret);
}

static void
long_invoke(Test test, CORBA_long dat, CORBA_Environment *ev)
{
	CORBA_long		ret;

	printf("request data = %d\n", (CORBA_int)dat);
	ret = test_func_long(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %d\n", (CORBA_int)ret);
}

static void
long_long_invoke(Test test, CORBA_long_long dat, CORBA_Environment *ev)
{
	CORBA_long_long		ret;

	printf("request data = %d\n", (CORBA_int)dat);
	ret = test_func_long_long(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %d\n", (CORBA_int)ret);
}

static void
float_invoke(Test test, CORBA_float dat, CORBA_Environment *ev)
{
	CORBA_float		ret;

	printf("request data = %f\n", dat);
	ret = test_func_float(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %f\n", ret);
}

static void
double_invoke(Test test, CORBA_double dat, CORBA_Environment *ev)
{
	CORBA_double		ret;

	printf("request data = %f\n", dat);
	ret = test_func_double(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((ret == dat + 1), (" "));
	printf("reply data = %f\n", ret);
}

static void
string_invoke(Test test, CORBA_char *str, CORBA_char *chk_str, CORBA_Environment *ev)
{
	CORBA_char		*ret;

	printf("request data = %s\n", str);
	ret = test_func_string(test, str, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));
	TP_CHECK((strcmp(ret, chk_str) == 0), (" "));
	printf("reply data = %s\n", ret);
	CORBA_free(ret);
}

static void
Carray_invoke(Test test, CORBA_char start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_Carray		dat;
	test_Carray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %d\n", (CORBA_int)i, dat[i]);
	}
	ret = test_func_Carray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %d\n", (CORBA_int)i, ret[i]);
	}
	CORBA_free(ret);
}

static void
Sarray_invoke(Test test, CORBA_short start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_Sarray		dat;
	test_Sarray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %d\n", (CORBA_int)i, dat[i]);
	}
	ret = test_func_Sarray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %d\n", (CORBA_int)i, ret[i]);
	}
	CORBA_free(ret);
}

static void
Larray_invoke(Test test, CORBA_long start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_Larray		dat;
	test_Larray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %d\n", (CORBA_int)i, (CORBA_int)dat[i]);
	}
	ret = test_func_Larray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %d\n", (CORBA_int)i, (CORBA_int)ret[i]);
	}
	CORBA_free(ret);
}

static void
LLarray_invoke(Test test, CORBA_long_long start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_LLarray		dat;
	test_LLarray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %d\n", (CORBA_int)i, (CORBA_int)dat[i]);
	}
	ret = test_func_LLarray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %d\n", (CORBA_int)i, (CORBA_int)ret[i]);
	}
	CORBA_free(ret);
}

static void
Farray_invoke(Test test, CORBA_float start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_Farray		dat;
	test_Farray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %f\n", (CORBA_int)i, dat[i]);
	}
	ret = test_func_Farray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %f\n", (CORBA_int)i, ret[i]);
	}
	CORBA_free(ret);
}

static void
Darray_invoke(Test test, CORBA_double start_dat, CORBA_int size, CORBA_Environment *ev)
{
	test_Darray		dat;
	test_Darray_slice	*ret;
	CORBA_long		i;

	for (i = 0; i < size; i++) {
		dat[i] = start_dat + i;
		printf("request data[%d] = %f\n", (CORBA_int)i, dat[i]);
	}
	ret = test_func_Darray(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	for (i = 0; i < size; i++) {
		TP_CHECK((ret[i] == i + start_dat + size), (" "));
		printf("reply data[%d] = %f\n", (CORBA_int)i, ret[i]);
	}
	CORBA_free(ret);
}

void
struct_invoke(Test test, test_struct *dat, CORBA_Environment *ev)
{
	test_struct		*ret;

	printf("request dat->char = %d\n", dat->char_dat);
	printf("request dat->short = %d\n", dat->short_dat);
	printf("request dat->long = %d\n", (CORBA_int)dat->long_dat);
	printf("request dat->long_long = %d\n", (CORBA_int)dat->long_long_dat);
	printf("request dat->float = %f\n", dat->float_dat);
	printf("request dat->double = %f\n", dat->double_dat);

	ret = test_func_struct(test, dat, ev);
	TP_CHECK((ev->_major == CORBA_NO_EXCEPTION), (" "));

	TP_CHECK((ret->char_dat == dat->char_dat + 1), (" "));
	TP_CHECK((ret->short_dat == dat->short_dat + 1), (" "));
	TP_CHECK((ret->long_dat == dat->long_dat + 1), (" "));
	TP_CHECK((ret->long_long_dat == dat->long_long_dat + 1), (" "));
	TP_CHECK((ret->float_dat == dat->float_dat + 1), (" "));
	TP_CHECK((ret->double_dat == dat->double_dat + 1), (" "));
		
	printf("reply dat->char = %d\n", ret->char_dat);
	printf("reply dat->short = %d\n", ret->short_dat);
	printf("reply dat->long = %d\n", (CORBA_int)ret->long_dat);
	printf("reply dat->long_long = %d\n", (CORBA_int)ret->long_long_dat);
	printf("reply dat->float = %f\n", ret->float_dat);
	printf("reply dat->double = %f\n", ret->double_dat);

	CORBA_free(ret);
}

void
tsk(void)
{
	int			argc;
	char			*argv;
	CORBA_Environment	ev;
	CORBA_ORB		orb;
	Test			test;

	CORBA_exception_init(&ev);

	argc = 0;
	orb = CORBA_ORB_init(&argc, &argv, "e-corba", &ev);
	TP_CHECK((orb != CORBA_NULL), (" "));

	test = CORBA_ORB_string_to_object(orb, IORstr, &ev);
	TP_CHECK((test != CORBA_NULL), (" "));
	
	TP_LINE();
	printf("s210401 : START\n");
	printf("s210501 : START\n");
	{
		CORBA_int	i;

		IPA_LOG_init(0);

		for (i = 0; i < CHAR_DATA_NUM; i++) {
			char_invoke(test, char_chkdat[i], &ev);
		}		

		IPA_LOG_init(0);
	}
	printf("s210401 : OK\n");
	printf("s210501 : OK\n");

	TP_LINE();
	printf("s210402 : START\n");
	printf("s210502 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < SHORT_DATA_NUM; i++) {
			short_invoke(test, short_chkdat[i], &ev);
		}		
	}
	printf("s210402 : OK\n");
	printf("s210502 : OK\n");

	TP_LINE();
	printf("s210403 : START\n");
	printf("s210503 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < LONG_DATA_NUM; i++) {
			long_invoke(test, long_chkdat[i], &ev);
		}		
	}
	printf("s210403 : OK\n");
	printf("s210503 : OK\n");

	TP_LINE();
	printf("s210404 : START\n");
	printf("s210504 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < LONGLONG_DATA_NUM; i++) {
			long_long_invoke(test, long_long_chkdat[i], &ev);
		}		
	}
	printf("s210404 : OK\n");
	printf("s210504 : OK\n");

	TP_LINE();
	printf("s210405 : START\n");
	printf("s210505 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < FLOAT_DATA_NUM; i++) {
			float_invoke(test, float_chkdat[i], &ev);
		}		
	}
	printf("s210405 : OK\n");
	printf("s210505 : OK\n");

	TP_LINE();
	printf("s210406 : START\n");
	printf("s210506 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < DOUBLE_DATA_NUM; i++) {
			double_invoke(test, double_chkdat[i], &ev);
		}		
	}
	printf("s210406 : OK\n");
	printf("s210506 : OK\n");

	TP_LINE();
	printf("s210407 : START\n");
	printf("s210507 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < STRING_DATA_NUM; i++) {
			string_invoke(test, string_chkdat[i].request,
				      string_chkdat[i].reply, &ev);
		}		
	}
	printf("s210407 : OK\n");
	printf("s210507 : OK\n");

	TP_LINE();
	printf("s210408 : START\n");
	printf("s210508 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < C_ARRAY_DATA_NUM; i++) {
			Carray_invoke(test, carray_chkdat[i].dat,
				      carray_chkdat[i].size, &ev);
		}		
	}
	printf("s210408 : OK\n");
	printf("s210508 : OK\n");

	TP_LINE();
	printf("s210409 : START\n");
	printf("s210509 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < S_ARRAY_DATA_NUM; i++) {
			Sarray_invoke(test, sarray_chkdat[i].dat,
				      sarray_chkdat[i].size, &ev);
		}		
	}
	printf("s210409 : OK\n");
	printf("s210509 : OK\n");

	TP_LINE();
	printf("s210410 : START\n");
	printf("s210510 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < L_ARRAY_DATA_NUM; i++) {
			Larray_invoke(test, larray_chkdat[i].dat,
				      larray_chkdat[i].size, &ev);
		}		
	}
	printf("s210410 : OK\n");
	printf("s210510 : OK\n");

	TP_LINE();
	printf("s210411 : START\n");
	printf("s210511 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < LL_ARRAY_DATA_NUM; i++) {
			LLarray_invoke(test, llarray_chkdat[i].dat,
				      llarray_chkdat[i].size, &ev);
		}		
	}
	printf("s210411 : OK\n");
	printf("s210511 : OK\n");

	TP_LINE();
	printf("s210412 : START\n");
	printf("s210512 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < F_ARRAY_DATA_NUM; i++) {
			Farray_invoke(test, farray_chkdat[i].dat,
				      farray_chkdat[i].size, &ev);
		}		
	}
	printf("s210412 : OK\n");
	printf("s210512 : OK\n");

	TP_LINE();
	printf("s210413 : START\n");
	printf("s210513 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < D_ARRAY_DATA_NUM; i++) {
			Darray_invoke(test, darray_chkdat[i].dat,
				      darray_chkdat[i].size, &ev);
		}		
	}
	printf("s210413 : OK\n");
	printf("s210513 : OK\n");

	TP_LINE();
	printf("s210414 : START\n");
	printf("s210514 : START\n");
	{
		CORBA_int	i;

		for (i = 0; i < STRUCT_DATA_NUM; i++) {
			struct_invoke(test, &struct_chkdat[i], &ev);
		}
	}
	printf("s210414 : OK\n");
	printf("s210514 : OK\n");
	TP_LINE();

	CORBA_Object_release(test, &ev);
	CORBA_Object_release((CORBA_Object)orb, &ev);

	exit(0);
}

#ifdef	CORBA_UNIX

void
main(int argc, char *argv[])
{
	SYS_init_debug(0);
	if (ecorba_init()) {
		printf("ERROR ecorba_init\n");
		exit(-1);
	}

	IORstr = argv[1];

	tsk();
	exit(0);
}

#else	/* CORBA_UNIX */

ERR
main(Bool Startup, TC *arg)
{
	WERR		tskid0;

	SYS_init_debug(0);
	IPA_LOG_init(0);

	TP_corba_init();
	if (TP_IOR_recv(6000, IORstr) == CORBA_FALSE) {
		return -1;
	}

	tskid0 = TP_CreateTask(tsk, DEFAULT_PRI);

	return 0;
}

#endif	/* CORBA_UNIX */
